#!/bin/bash
#
# backup-rotate
# delete the oldest files from a set, only leave a desired amount
# Copyright (c) 2006-2008, Matej Sustr
# 
# $Id: backup-rotate.sh 29 2009-09-17 18:48:42Z matej $
VERSION="1.7"
COPYRIGHT="Copyright (c) 2006-2009, Matej Sustr"
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

usage() { [ "${LANG:0:2}" != "sk" ] && cat <<. | cut -c 3-
# 
# backup-rotate v$VERSION - delete old files     $COPYRIGHT
# ARGUMENTS:
# [-v] [-d <days>] [-n <nfiles>] <prefix1> [... <prefixN>]
#  	-v		(verbose) prints every file name deletion
# 	-d <days>	only delete files older than 'days' days
# 			(default = $defdays)
# 	-n <nfiles>	regardless of their age, only delete files so that
# 			at least 'nfiles' are left
# 			(default = $defnfiles)
# 	<prefixK>	prefix of the group of files to delete from
#
# EXAMPLE:
#   $0 -v /var/recordings/  # directories must have a slash!
#   $0 -d 10 /tmp/phpsess-
#   $0 -d 5 -n 5 /var/log/messages. /var/log/syslog.
# 
.
[ "${LANG:0:2}" = "sk" ] && cat <<. | cut -c 3-
# 
# backup-rotate v$VERSION - zmaz stare subory    $COPYRIGHT
# PARAMETRE:
# [-v] [-d <days>] [-n <nfiles>] <prefix1> [... <prefixN>]
#  	-v		(verbose) vypise kazdy subor, ktory sa bude mazat
# 	-d <days>	bude mazat iba subory starsie ako 'days' dni
# 			(default = $defdays)
# 	-n <nfiles>	bez ohladu na cas, maze subory vzdy tak, aby zostalo
# 			aspon 'nfiles' suborov
# 			(default = $defnfiles)
# 	<prefixK>	prefixy skupiny suborov, ktora sa uvazuje na premazanie
#
# NAPRIKLAD:
#   $0 -v /var/recordings/  # adresar musi mat vzdy lomitko!
#   $0 -d 10 /tmp/phpsess-
#   $0 -d 5 -n 5 /var/log/messages. /var/log/syslog.
# 
.
}
# REQUIRED:
# find, sed, rm, sort
# 
# KNOWN BUGS:
# - undefined behavior if any of the matching files contain \n in their name
# 

MYSELF=backup-rotate.sh
PATH=/usr/local/bin:/usr/bin:/bin	# be safe

##############################################################################

function require_number {
	[ "$OPTARG" -eq "$OPTARG" 2>/dev/null ] || {
		echo >&2 "-$opt needs a number"
		usage
		exit 1
	}
}

# defaults:
verbose=no
defdays=180;	days=$defdays
defnfiles=6;	nfiles=$defnfiles
while getopts "hvd:n:" opt; do case $opt in
	v)	verbose=yes;;
	d)	require_number
		days=$OPTARG;;
	n)	require_number
		nfiles=$OPTARG;;
	h|\?|*)	usage; exit 1;;
esac done
shift $((OPTIND-1))
if [ $# -eq 0 ]; then
	usage; exit 1
fi
if [ "$days" -lt 0 ]; then
	mtime=""		# allow today's files to be deleted
else
	mtime="-mtime +$days"
fi
if [ "$nfiles" -lt 1 ]; then
	nfilesd=""
else
	nfilesd="1,${nfiles}d;"	# keep newest N files
fi


function verbose {
	[ "$verbose" = "yes" ] && return 0 || return 1
}

while [ "$1" != "" ]; do
	prefixdir="$(echo "$1" | sed 's/[^\/]*$//')"
	if [ "$prefixdir" = "" ]; then
		prefixdir="."
	fi
	prefixfile="$(echo "$1" | sed 's/.*\///')"
	shift

	find "$prefixdir" -maxdepth 1 -type f		\
	    -name "$prefixfile"\* -printf '%T@-%p\n'	\
	  | sort -rn					\
	  | sed "${nfilesd}s/^[0-9.,]*-//"		\
	  | while read f; do
#		echo "DEBUG: delfile='$f'"
		[ "`find "$f" $mtime`" = "$f" ] && {
			verbose && echo >&2 "$MYSELF: Deleting '$f'."
			rm -f "$f" || echo >&2 "$MYSELF: Error deleting '$f'."
		}
	done
done
